using System;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.SpaServices;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.IdentityModel.Tokens;
using TEAMModelOS.SDK.Context.Configuration;
using TEAMModelOS.SDK.Context.Filter;
using TEAMModelOS.SDK.Extension.JwtAuth;
using TEAMModelOS.SDK.Module.AzureBlob.Configuration;
using TEAMModelOS.SDK.Module.AzureCosmosDB.Configuration;
using TEAMModelOS.SDK.Module.AzureTable.Configuration;
using VueCliMiddleware;

namespace TEAMModelOS
{
    public class Startup
    {
        readonly string MyAllowSpecificOrigins = "_myAllowSpecificOrigins";
        public Startup(IConfiguration configuration, IWebHostEnvironment env)
        {
            Configuration = configuration;
            BaseConfigModel.SetBaseConfig(Configuration, env.ContentRootPath, env.WebRootPath);
        }

        public IConfiguration Configuration { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            // trueĬJr£ӳ䌢fʽӳQm^SAMLóRoleClaimType = 'http://schemas.microsoft.com/ws/2008/06/identity/claims/role'
            // falseRoleClaimType = 'roles' 
            /*
            JwtSecurityTokenHandler.DefaultMapInboundClaims = false;
            //@OValidAudiencesȌʂҪrAzure CN OAD耣һc̎]PSL녷ՕrClient耫@ȡˢ´a֮ṩ
            var tokenvalidationparameters = new TokenValidationParameters
            {
                RoleClaimType = "roles",
                ValidAudiences = new string[] { "8768b06f-c5c5-4b0c-abfb-d7ded354626d", "api://8768b06f-c5c5-4b0c-abfb-d7ded354626d" },
            }; 
            services.AddAuthentication(options =>
            {
                options.DefaultScheme = JwtBearerDefaults.AuthenticationScheme;
            }).AddJwtBearer(options => //AzureADJwtBearer
            {
                //options.SaveToken = true; //CɷɲЧm춷؆ֲʽܘ
                options.Authority = "https://login.microsoftonline.com/73a2bcc5-fe99-4566-aa8a-07e7bb287df1/v2.0";
                options.RequireHttpsMetadata = true;
                options.Audience = "8768b06f-c5c5-4b0c-abfb-d7ded354626d";
                options.TokenValidationParameters = tokenvalidationparameters;
                options.Events = new JwtBearerEvents();
                //¼Ҫo䛄t_
                //options.Events.OnMessageReceived = async context => { await Task.FromResult(0); };
                //options.Events.OnForbidden = async context => { await Task.FromResult(0); };
                //options.Events.OnChallenge = async context => { await Task.FromResult(0); };
                //options.Events.OnAuthenticationFailed = async context => { await Task.FromResult(0); };
                options.Events.OnTokenValidated = async context =>
                {
                    if (!context.Principal.Claims.Any(x => x.Type == "http://schemas.microsoft.com/identity/claims/scope") //ClaimConstants.Scope
                       && !context.Principal.Claims.Any(y => y.Type == "roles")) //ClaimConstants.Roles //http://schemas.microsoft.com/ws/2008/06/identity/claims/role
                    {
                        //TODO ̎~ڙǽɫLo
                        throw new UnauthorizedAccessException("Neither scope or roles claim was found in the bearer token.");
                    }
                    await Task.FromResult(0);
                };
            });
            */
            //OՈ
            //services.AddCors(options =>
            //{
            //    options.AddPolicy(MyAllowSpecificOrigins,
            //    builder =>
            //    {
            //        builder.WithOrigins("http://example.com",
            //                            "http://www.contoso.com")
            //        .AllowAnyHeader()
            //        .AllowAnyMethod(); 
            //    });
            //});
            services.AddMemoryCache();
            services.AddControllers();
            // Table
            services.AddAzureTableStorage().AddConnection(Configuration.GetSection("Azure:Table").Get<AzureTableOptions>());
            //ʹBlob
            services.AddAzureBlobStorage().AddConnection(Configuration.GetSection("Azure:Blob").Get<AzureBlobOptions>());
            //ʹCosmosDB
            services.AddAzureCosmosDB().AddCosmosDBConnection(Configuration.GetSection("Azure:CosmosDB").Get<AzureCosmosDBOptions>());
            //HttpContextAccessorHttpContext
            services.AddSingleton<IHttpContextAccessor, HttpContextAccessor>();
            //Jwt
            services.JwtAuth(Configuration.GetSection("JwtSetting"));
            //ȫɨIBusinessServiceӿڵʵ
            services.Scan(scan => scan.FromApplicationDependencies()
               .AddClasses(classes => classes.AssignableTo<IBusinessService>())
                   .AsImplementedInterfaces()
                   .WithSingletonLifetime());
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }
            //Ҫdg
            app.UseStaticFiles();
            //app.UseSpaStaticFiles(); //ʹg_
            app.UseRouting();
            app.UseMiddleware<HttpGlobalExceptionInvoke>();
            //app.UseCors(MyAllowSpecificOrigins); //ʹÿO
            //app.UseHttpsRedirection(); //_lЕrP
            app.UseAuthentication();
            app.UseAuthorization();
            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
#if DEBUG
                endpoints.MapToVueCliProxy(
                    "{*path}",
                    new SpaOptions { SourcePath = "ClientApp", },
                    npmScript: (System.Diagnostics.Debugger.IsAttached) ? "serve" : null
                    );
#else
                endpoints.MapFallbackToFile("index.html");
#endif

                //endpoints.MapGet("/", async context =>
                //{
                //    await context.Response.WriteAsync("Hello World!");
                //});

            });
        }
    }
}
